@@ -39,6 +39,8 @@ import logging
 
 import numpy as np
 from pyresample import image, utils, geometry, kd_tree
+from pyresample.bilinear import get_sample_from_bil_info, get_bil_info
+
 from mpop import CONFIG_PATH
 
 logger = logging.getLogger(__name__)
@@ -93,11 +95,15 @@ class Projector(object):
     generated projectors can be saved to disk for later reuse. Use the
     :meth:`save` method for this.
 
-    To define a projector object, on has to specify *in_area* and *out_area*,
-    and can also input the *in_lonlats* or the *mode* ('quick' which works only
-    if both in- and out-areas are AreaDefinitions, or 'nearest'). *radius*
-    defines the radius of influence for nearest neighbour search in 'nearest'
-    mode.
+    To define a projector object, on has to specify *in_area* and
+    *out_area*, and can also input the *in_lonlats* or the *mode*.
+    Available modes area:
+    - 'quick' (works only if both in- and out-areas are AreaDefinitions)
+    - 'bilinear' (out-area needs to be AreaDefinition with proj4_string)
+    - 'ewa'
+    - 'nearest'.
+    *radius* defines the radius of influence for nearest neighbour
+    search in 'nearest' and 'bilinear' modes.
     """
 
     def __init__(self, in_area, out_area,
@@ -105,8 +111,9 @@ class Projector(object):
                  radius=10000, nprocs=1):
 
         if (mode is not None and
-                mode not in ["quick", "nearest"]):
-            raise ValueError("Projector mode must be 'nearest' or 'quick'")
+                mode not in ["quick", "nearest", "ewa", "bilinear"]):
+            raise ValueError("Projector mode must be one of 'nearest', "
+                             "'quick', 'ewa', 'bilinear'")
 
         self.area_file = get_area_file()
 
@@ -133,6 +140,7 @@ class Projector(object):
                 self.in_area = in_area
             except AttributeError:
                 try:
+                    # TODO: Note that latlons are in order (lons, lats)
                     self.in_area = geometry.SwathDefinition(lons=in_latlons[0],
                                                             lats=in_latlons[1])
                     in_id = in_area
@@ -160,7 +168,7 @@ class Projector(object):
                                          self.area_file + " or "
                                          "be an area object.")
 
-        #if self.in_area == self.out_area:
+        # if self.in_area == self.out_area:
         #    return
 
         # choosing the right mode if necessary
@@ -216,6 +224,29 @@ class Projector(object):
                 self._cache['row_idx'] = ridx
                 self._cache['col_idx'] = cidx
 
+            elif self.mode == "ewa":
+                from pyresample.ewa import ll2cr
+                swath_points_in_grid, cols, rows = ll2cr(self.in_area,
+                                                         self.out_area)
+                self._cache = {}
+                # self._cache['ewa_swath_points_in_grid'] = \
+                #     swath_points_in_grid
+                self._cache['ewa_cols'] = cols
+                self._cache['ewa_rows'] = rows
+
+            elif self.mode == "bilinear":
+
+                bilinear_t, bilinear_s, input_idxs, idx_arr = \
+                    get_bil_info(self.in_area, self.out_area,
+                                 self.radius, neighbours=32,
+                                 nprocs=nprocs, masked=False)
+
+                self._cache = {}
+                self._cache['bilinear_s'] = bilinear_s
+                self._cache['bilinear_t'] = bilinear_t
+                self._cache['input_idxs'] = input_idxs
+                self._cache['idx_arr'] = idx_arr
+
     def save(self, resave=False):
         """Save the precomputation to disk, and overwrite existing file in case
         *resave* is true.
@@ -258,4 +289,33 @@ class Projector(object):
             res = np.ma.array(img.get_array_from_linesample(row_idx, col_idx),
                               dtype=data.dtype)
 
+        elif self.mode == "ewa":
+            from pyresample.ewa import fornav
+            # TODO: should be user configurable?
+            rows_per_scan = None
+
+            if 'ewa_cols' not in self._cache:
+                self._cache['ewa_cols'] = self._file_cache['ewa_cols']
+                self._cache['ewa_rows'] = self._file_cache['ewa_rows']
+            num_valid_points, res = fornav(self._cache['ewa_cols'],
+                                           self._cache['ewa_rows'],
+                                           self.out_area, data,
+                                           rows_per_scan=rows_per_scan)
+
+        elif self.mode == "bilinear":
+
+            if 'bilinear_t' not in self._cache:
+                self._cache['bilinear_t'] = self._file_cache['bilinear_t']
+                self._cache['bilinear_s'] = self._file_cache['bilinear_s']
+                self._cache['input_idxs'] = self._file_cache['input_idxs']
+                self._cache['idx_arr'] = self._file_cache['idx_arr']
+
+            res = get_sample_from_bil_info(data.ravel(),
+                                           self._cache['bilinear_t'],
+                                           self._cache['bilinear_s'],
+                                           self._cache['input_idxs'],
+                                           self._cache['idx_arr'],
+                                           output_shape=self.out_area.shape)
+            res = np.ma.masked_invalid(res)
+
         return res
